#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "meter.h"
#include "meter_tools.h"


void display_config_sane() {
  /* set all 4 chips to dynamic mode, noblank */
  /*    notest, 12mA current, blank all segments */ 
  emulate_i2c("\x70\x00\x47\x00\x00\x00\x00",7);
  emulate_i2c("\x72\x00\x47\x00\x00\x00\x00",7);
  emulate_i2c("\x74\x00\x47\x00\x00\x00\x00",7);
  emulate_i2c("\x76\x00\x47\x00\x00\x00\x00",7);
}

unsigned short existing_state[8]= {
   0x0,0x0,0x0,0x0,
   0x0,0x0,0x0,0x0,
};

static unsigned short reverse_bits16(unsigned short v) {
      
      unsigned int s = 16;
      unsigned int mask = ~0;         
      while ((s >>= 1) > 0) {
	       mask ^= (mask << s);
	       v = ((v >> s) & mask) | ((v << s) & ~mask);
      }
     
      return v;
}



void update_display(unsigned short in_state[8]) {

  int i;
  char buffer[6];
  int address=0x70;
  unsigned short temp;
  unsigned short state[8];
   
   /* c passes arrays by reference so we have to do this         */ 
   /* otherwise changes to state[] are propogated back to caller */
  for(i=0;i<8;i++) state[i]=in_state[i];
   
  /* state[0]=digit0 */
  /* state[1]=digit1 */
  /* state[2]=digit2 */
  /* state[3]=digit3 */
  /* state[4]=digit4 */
  /* state[5]=digit5 */
  /* state[6]=left bargraph */
  /* state[7]=right bargraph */
   
  for(i=0;i<4;i++) {
  
     /* the bargraph was wired backward on actual hardware */
     /* fix it here to make software easier to write       */
     if (i==3) {
	
	temp=(reverse_bits16(state[6])<<6)|(state[6]&0x3f);
	state[6]=temp;
	
	temp=(reverse_bits16(state[7])<<6)|(state[7]&0x3f);
	state[7]=temp;
       	
     }
     
     /* only update if there's been a change */
     if ( (existing_state[i*2]!=state[i*2]) ||
	  (existing_state[(i*2)+1]!=state[(i*2)+1])) {
	
	existing_state[i*2]=state[i*2];
	existing_state[(i*2)+1]=state[(i*2)+1];
     
	/* write out to hardware */
	
        buffer[0]=address+(i*2);
        buffer[1]=0x01;
        buffer[2]=(state[i*2]>>8)&0xff;
        buffer[3]=(state[(i*2)+1]>>8)&0xff;
        buffer[4]=(state[i*2])&0xff;
        buffer[5]=(state[(i*2)+1])&0xff;
        emulate_i2c(buffer,6);
     }
  }
}

/* I did this by hand                                    */
/* then touched up some of the lousy looking ones        */
/*   based on ideas from the MAX6954 chip docs           */
/*   as well as from                                     */
/* http://www.robotroom.com/MultiSegmentLEDDisplay3.html */
/* This is probably still sub-optimal.  I want each char */
/*   to be distinct, so that when looking at something   */
/*   like a code listing you can tell the difference     */
/*   between C and [, as well as 1 : l ! |               */
short ascii_lookup[256]={

  /* 0x00 */ 0,
  /* 0x01 */ 0,
  /* 0x02 */ 0,
  /* 0x03 */ 0,
  /* 0x04 */ 0,
  /* 0x05 */ 0,
  /* 0x06 */ 0,
  /* 0x07 */ 0,
  /* 0x08 */ 0,
  /* 0x09 */ 0,
  /* 0x0a */ 0,
  /* 0x0b */ 0,
  /* 0x0c */ 0,
  /* 0x0d */ 0,
  /* 0x0e */ 0,
  /* 0x0f */ 0,
  /* 0x10 */ 0,
  /* 0x11 */ 0,
  /* 0x12 */ 0,
  /* 0x13 */ 0,
  /* 0x14 */ 0,
  /* 0x15 */ 0,
  /* 0x16 */ 0,
  /* 0x17 */ 0,
  /* 0x18 */ 0,
  /* 0x19 */ 0,
  /* 0x1a */ 0,
  /* 0x1b */ 0,
  /* 0x1c */ 0,
  /* 0x1d */ 0,
  /* 0x1e */ 0,
  /* 0x1f */ 0,
  /* 0x20 ' ' */ 0,
  /* 0x21 '!' */ 0x9124,    /* 1001 0001 0010 0100 adhlp      */
  /* 0x22 '"' */ 0x4200,    /* 0100 0010 0000 000x bg         */
  /* 0x23 '#' */ 0x62a8,    /* 0110 0010 1010 100x bcgjln     */
  /* 0x24 '$' */ 0xb6a8,    /* 1011 0110 1010 100x acdfgjln   */
  /* 0x25 '%' */ 0x2510,    /* 0010 0101 0001 000x cfhm       */
  /* 0x26 '&' */ 0x994c,    /* 1001 1001 0100 110x adehknp    */
  /* 0x27 ''' */ 0x0100,    /* 0000 0001 0000 000x h          */
  /* 0x28 '(' */ 0x0140,    /* 0000 0001 0100 000x hk         */
                            /* abcd efgh jklm np.X            */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x29 ')' */ 0x0014,    /* 0000 0000 0001 010x mp */
  /* 0x2a '*' */ 0x03fc,    /* 0000 0011 1111 110x ghjklmnp */
  /* 0x2b '+' */ 0x02a8,    /* 0000 0010 1010 100x gjln */
  /* 0x2c ',' */ 0x0010,    /* 0000 0000 0001 000x m */
  /* 0x2d '-' */ 0x0088,    /* 0000 0000 1000 100x jn */
  /* 0x2e '.' */ 0x0040,    /* 0000 0000 0100 000x k */
  /* 0x2f '/' */ 0x0110,    /* 0000 0001 0001 000x hm */
  /* 0x30 '0' */ 0xfd10,    /* 1111 1101 0001 000x abcdefhm */
  /* 0x31 '1' */ 0x6100,    /* 0110 0001 0000 000x bch */
  /* 0x32 '2' */ 0xd888,    /* 1101 1000 1000 100x abdejn */
  /* 0x33 '3' */ 0xf088,    /* 1111 0000 1000 100x abcdjn */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x34 '4' */ 0x6488,    /* 0110 0100 1000 100x bcfjn */
  /* 0x35 '5' */ 0x9448,    /* 1001 0100 0100 100x adfkn */
  /* 0x36 '6' */ 0xbc88,    /* 1011 1100 1000 100x acdefjn */
  /* 0x37 '7' */ 0x81a8,    /* 1000 0001 1010 100x ahjln */
  /* 0x38 '8' */ 0xfc88,    /* 1111 1100 1000 100x abcdefjn */
  /* 0x39 '9' */ 0xe488,    /* 1110 0100 1000 100x abcfjn */
  /* 0x3a ':' */ 0x0220,    /* 0000 0010 0010 000x gl */
  /* 0x3b ';' */ 0x0210,    /* 0000 0010 0001 000x gm */
  /* 0x3c '<' */ 0x1110,    /* 0001 0001 0001 000x dhm */
  /* 0x3d '=' */ 0x1088,    /* 0001 0000 1000 100x djn */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x3e '>' */ 0x1044,    /* 0001 0000 0100 010x dkp      */
  /* 0x3f '?' */ 0xc4a0,    /* 1100 0100 1010 000x abfjl    */
  /* 0x40 '@' */ 0xde80,    /* 1101 1110 1000 000x abdefgj  */
  /* 0x41 'A' */ 0xec88,    /* 1110 1100 1000 100x abcefjn  */
  /* 0x42 'B' */ 0x9d48,    /* 1001 1101 0100 100x adefhkn  */
  /* 0x43 'C' */ 0x9c00,    /* 1001 1100 0000 000x adef     */
  /* 0x44 'D' */ 0xf220,    /* 1111 0010 0010 000x abcdgl   */
  /* 0x45 'E' */ 0x9c88,    /* 1001 1100 1000 100x adefjn   */
  /* 0x46 'F' */ 0x8c88,    /* 1000 1100 1000 100x aefjn    */
  /* 0x47 'G' */ 0xbc80,    /* 1011 1100 1000 000x acdefj   */
  /* 0x48 'H' */ 0x6c88,    /* 0110 1100 1000 100x bcefnj   */
  /* 0x49 'I' */ 0x9220,    /* 1001 0010 0010 000x adgl     */
  /* 0x4a 'J' */ 0x7800,    /* 0111 1000 0000 000x bcde     */
  /* 0x4b 'K' */ 0x0d48,    /* 0000 1101 0100 100x efhkn    */
  /* 0x4c 'L' */ 0x1c00,    /* 0001 1100 0000 000x def      */
  /* 0x4d 'M' */ 0x6d04,    /* 0110 1101 0000 010x bcefhp   */
  /* 0x4e 'N' */ 0x6c44,    /* 0110 1100 0100 010x bcefkp   */
  /* 0x4f 'O' */ 0xfc00,    /* 1111 1100 0000 000x abcdef   */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x50 'P' */ 0xcc88,    /* 1100 1100 1000 100x abefjn     */
  /* 0x51 'Q' */ 0xfc40,    /* 1111 1100 0100 000x abcdefk    */
  /* 0x52 'R' */ 0xccc8,    /* 1100 1100 1100 100x abefjkn    */
  /* 0x53 'S' */ 0xb488,    /* 1011 0100 1000 100x acdfjn     */
  /* 0x54 'T' */ 0x8220,    /* 1000 0010 0010 000x agl        */
  /* 0x55 'U' */ 0x7c00,    /* 0111 1100 0000 000x bcdef      */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x56 'V' */ 0x0d10,    /* 0000 1101 0001 000x efhm     */
  /* 0x57 'W' */ 0x6c50,    /* 0110 1100 0101 000x bcefkm   */
  /* 0x58 'X' */ 0x0154,    /* 0000 0001 0101 010x hkmp     */
  /* 0x59 'Y' */ 0x0124,    /* 0000 0001 0010 010x hlp      */
  /* 0x5a 'Z' */ 0x9110,    /* 1001 0001 0001 000x adhm     */
  /* 0x5b '[' */ 0x901c,    /* 1001 0000 0001 110x admnp    */
  /* 0x5c '\' */ 0x0044,    /* 0000 0000 0100 010x kp       */
  /* 0x5d ']' */ 0x91c0,    /* 1001 0001 1100 000x adhjk    */
  /* 0x5e '^' */ 0x4100,    /* 0100 0001 0000 000x bh       */
  /* 0x5f '_' */ 0x1000,    /* 0001 0000 0000 000x d        */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x60 '`' */ 0x0004,    /* 0000 0000 0000 010x p     */
  /* 0x61 'a' */ 0xf888,    /* 1111 1000 1000 100x abcdejn    */
  /* 0x62 'b' */ 0x3c88,    /* 0011 1100 1000 100x cdefjn    */
  /* 0x63 'c' */ 0x1888,    /* 0001 1000 1000 100x dejn     */
  /* 0x64 'd' */ 0x7888,    /* 0111 1000 1000 100x bcdejn        */
  /* 0x65 'e' */ 0xdc88,    /* 1101 1100 1000 100x abdefjn     */
  /* 0x66 'f' */ 0x01a8,    /* 0000 0001 1010 100x hjln   */
  /* 0x67 'g' */ 0xf084,    /* 1111 0000 1000 010x abcdjp   */
  /* 0x68 'h' */ 0x2c88,    /* 0010 1100 1000 100x cefjn     */
  /* 0x69 'i' */ 0x0020,    /* 0000 0000 0010 000x l      */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x6a 'j' */ 0x7000,    /* 0111 0000 0000 000x bcd     */
  /* 0x6b 'k' */ 0x0360,    /* 0000 0011 0110 000x ghkl     */
  /* 0x6c 'l' */ 0x1220,    /* 0001 0010 0010 000x dgl       */
  /* 0x6d 'm' */ 0x28a8,    /* 0010 1000 1010 100x cejln     */
  /* 0x6e 'n' */ 0x0848,    /* 0000 1000 0100 100x ekn       */
  /* 0x6f 'o' */ 0x3888,    /* 0011 1000 1000 100x cdejn        */
  /* 0x70 'p' */ 0x8d08,    /* 1000 1101 0000 100x aefhn     */
  /* 0x71 'q' */ 0xe084,    /* 1110 0000 1000 010x abcjp    */
  /* 0x72 'r' */ 0x0888,    /* 0000 1000 1000 100x ejn        */
  /* 0x73 's' */ 0x10c0,    /* 0001 0000 1100 000x djk        */
  /* 0x74 't' */ 0x1c08,    /* 0001 1100 0000 100x defn       */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
  /* 0x75 'u' */ 0x3800,    /* 0011 1000 0000 000x cde      */
  /* 0x76 'v' */ 0x0810,    /* 0000 1000 0001 000x em       */
  /* 0x77 'w' */ 0x2850,    /* 0010 1000 0101 000x cekm     */
  /* 0x78 'x' */ 0x01dc,    /* 0000 0001 1101 110x hjkmnp   */
  /* 0x79 'y' */ 0x7280,    /* 0111 0010 1000 000x bcdgj    */
  /* 0x7a 'z' */ 0x1018,    /* 0001 0000 0001 100x dmn      */
  /* 0x7b '{' */ 0x01c0,    /* 0000 0001 1100 000x hjk      */
  /* 0x7c '|' */ 0x0c00,    /* 0000 1100 0000 000x ef       */
  /* 0x7d '}' */ 0x001c,    /* 0000 0000 0001 110x mnp      */
  /* 0x7e '~' */ 0x8000,    /* 1000 0000 0000 000x a        */
  /* 0x7f '' */  0xfffc,    /* 1111 1111 1111 110x          */
                            /* abcd efgh jklm np.X */
/*   A
    PGH
    ___
F  |\|/|  B
N   - -   J
E  |/|\|  C
    ---
    MLK
     D
*/
};
